'use strict';

var _interopRequireDefault = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

exports.__esModule = true;
exports['default'] = makeHotify;

var _React = require('react');

var _React2 = _interopRequireDefault(_React);

var _makeProxy = require('./makeProxy');

var _makeProxy2 = _interopRequireDefault(_makeProxy);

/**
 * Force-updates an instance regardless of whether
 * it descends from React.Component or not.
 */
function forceUpdate(instance) {
  _React2['default'].Component.prototype.forceUpdate.call(instance);
}

/**
 * Wraps componentWillMount and componentWillUnmount to
 * push and remove instances from `mountedInstances`.
 * This lets us `forceUpdate` instances when we need to.
 *
 * We could listen to componentDidMount, but shallow renderer
 * we're using in tests doesn't call it, and we don't really care.
 */
function trackMount(prototype, mountedInstances) {
  var realComponentWillMount = prototype.componentWillMount;
  prototype.componentWillMount = function componentWillMount() {
    mountedInstances.push(this);

    if (realComponentWillMount) {
      realComponentWillMount.apply(this, arguments);
    }
  };

  var realComponentWillUnmount = prototype.componentWillUnmount;
  prototype.componentWillUnmount = function componentWillUnmount() {
    mountedInstances.splice(mountedInstances.indexOf(this), 1);

    if (realComponentWillUnmount) {
      realComponentWillUnmount.apply(this, arguments);
    }
  };

  return prototype;
}

function makeHotify() {
  var proxyTo = _makeProxy2['default']({});
  var mountedInstances = [];
  var CurrentClass = null;

  function HotClass() {
    CurrentClass.apply(this, arguments);
  }

  return function hotify(NextClass) {
    CurrentClass = NextClass;

    // Wow, this is dense!
    // I have no idea what's going on here, but it works.
    HotClass.prototype = trackMount(proxyTo(NextClass.prototype), mountedInstances);
    HotClass.prototype.__proto__ = NextClass.prototype;
    HotClass.prototype.constructor = HotClass;
    HotClass.prototype.constructor.__proto__ = NextClass;

    mountedInstances.forEach(forceUpdate);

    return HotClass;
  };
}

module.exports = exports['default'];